@@ -1,32 +1,34 @@
 package Catalyst::View::Email::Template;
 
-use Moose;
+use warnings;
+use strict;
+
+use Class::C3;
 use Carp;
 use Scalar::Util qw/ blessed /;
-extends 'Catalyst::View::Email';
 
-our $VERSION = '0.26_01';
-$VERSION = eval $VERSION;
+use Email::MIME::Creator;
+
+use base qw/ Catalyst::View::Email /;
+
+our $VERSION = '0.09999_02';
+
 =head1 NAME
 
 Catalyst::View::Email::Template - Send Templated Email from Catalyst
 
 =head1 SYNOPSIS
 
-Sends templated mail, based upon your default view. It captures the output
+Sends Templated mail, based upon your default view. It captures the output
 of the rendering path, slurps in based on mime-types and assembles a multi-part
-email using L<Email::MIME::Creator> and sends it out.
+email using Email::MIME::Creator and sends it out.
 
 =head1 CONFIGURATION
 
-WARNING: since version 0.10 the configuration options slightly changed!
-
-Use the helper to create your view:
+Use the helper to create your View:
     
     $ script/myapp_create.pl view Email::Template Email::Template
 
-For basic configuration look at L<Catalyst::View::Email/CONFIGURATION>.
-
 In your app configuration (example in L<YAML>):
 
     View::Email::Template:
@@ -34,35 +36,49 @@ In your app configuration (example in L<YAML>):
         # template  paths.
         # Default: none
         template_prefix: email
+        # Where to look in the stash for the email information.
+        # Default: email
+        stash_key: email
         # Define the defaults for the mail
         default:
+            # Defines the default content type (mime type).
+            # Mandatory
+            content_type: text/html
+            # Defines the default charset for every MIME part with the content
+            # type text.
+            # According to RFC2049 a MIME part without a charset should
+            # be treated as US-ASCII by the mail client.
+            # If the charset is not set it won't be set for all MIME parts
+            # without an overridden one.
+            # Default: none
+            charset: utf-8
             # Defines the default view used to render the templates.
             # If none is specified neither here nor in the stash
             # Catalysts default view is used.
             # Warning: if you don't tell Catalyst explicit which of your views should
             # be its default one, C::V::Email::Template may choose the wrong one!
             view: TT
+        # Setup how to send the email
+        # All those options are passed directly to Email::Send,
+        # for all available options look at its docs.
+        sender:
+            mailer: SMTP
+            mailer_args:
+                Host:       smtp.example.com # defaults to localhost
+                username:   username
+                password:   password
 
 =head1 SENDING EMAIL
 
-Sending email works just like for L<Catalyst::View::Email> but by specifying 
-the template instead of the body and forwarding to your Email::Template view:
+Sending email is just setting up your defaults, the stash key and forwarding to the view.
 
-    sub controller : Private {
-        my ( $self, $c ) = @_;
-
-        $c->stash->{email} = {
-            to          => 'jshirley@gmail.com',
-            cc          => 'abraxxa@cpan.org',
-            bcc         => 'hidden@secret.com hidden2@foobar.com',
-            from        => 'no-reply@foobar.com',
-            subject     => 'I am a Catalyst generated email',
-            template    => 'test.tt',
-            content_type => 'multipart/alternative'
-        };
-        
-        $c->forward( $c->view('Email::Template') );
-    }
+    $c->stash->{email} = {
+        to      => 'jshirley@gmail.com',
+        from    => 'no-reply@foobar.com',
+        subject => 'I am a Catalyst generated email',
+        template => 'test.tt',
+    };
+    $c->forward('View::Email::Template');
 
 Alternatively if you want more control over your templates you can use the following idiom
 to override the defaults:
@@ -83,40 +99,17 @@ to override the defaults:
     ]
 
 
-=head1 HANDLING ERRORS
-
-See L<Catalyst::View::Email/HANDLING ERRORS>.
+If it fails $c->error will have the error message.
 
 =cut
 
 # here the defaults of Catalyst::View::Email are extended by the additional
 # ones Template.pm needs.
 
-has 'stash_key' => (
-    is      => 'rw',
-    isa     => 'Str',
-    default => sub { "email" },
-    lazy    => 1,
+__PACKAGE__->config(
+    template_prefix => '',
 );
 
-has 'template_prefix' => (
-    is      => 'rw',
-    isa     => 'Str',
-    default => sub { '' },
-    lazy    => 1,
-);
-
-has 'default' => (
-    is      => 'rw',
-    isa     => 'HashRef',
-    default => sub {
-        {
-            view         => 'TT',
-            content_type => 'text/html',
-        };
-    },
-    lazy => 1,
-);
 
 # This view hitches into your default view and will call the render function
 # on the templates provided.  This means that you have a layer of abstraction
@@ -136,90 +129,59 @@ has 'default' => (
 #multipart/alternative
 
 sub _validate_view {
-    my ( $self, $view ) = @_;
+    my ($self, $view) = @_;
+    
+    croak "Email::Template's configured view '$view' isn't an object!"
+        unless (blessed($view));
 
-    croak "C::V::Email::Template's configured view '$view' isn't an object!"
-      unless ( blessed($view) );
+    croak "Email::Template's configured view '$view' isn't an Catalyst::View!"
+        unless ($view->isa('Catalyst::View'));
 
-    croak
-      "C::V::Email::Template's configured view '$view' isn't an Catalyst::View!"
-      unless ( $view->isa('Catalyst::View') );
-
-    croak
-"C::V::Email::Template's configured view '$view' doesn't have a render method!"
-      unless ( $view->can('render') );
+    croak "Email::Template's configured view '$view' doesn't have a render method!"
+        unless ($view->can('render'));
 }
 
-=head1 METHODS
-
-=over 4
-
-=item generate_part
-
-Generates a MIME part to include in the email. Since the email is template based
-every template piece is a separate part that is included in the email.
-
-=cut
-
 sub generate_part {
-    my ( $self, $c, $attrs ) = @_;
+    my ($self, $c, $attrs) = @_;
 
-    my $template_prefix      = $self->template_prefix;
-    my $default_view         = $self->default->{view};
-    my $default_content_type = $self->default->{content_type};
-    my $default_charset      = $self->default->{charset};
+    my $template_prefix         = $self->{template_prefix};
+    my $default_view            = $self->{default}->{view};
+    my $default_content_type    = $self->{default}->{content_type};
+    my $default_charset         = $self->{default}->{charset};
 
     my $view;
-
     # use the view specified for the email part
-    if (   exists $attrs->{view}
-        && defined $attrs->{view}
-        && $attrs->{view} ne '' )
-    {
-        $view = $c->view( $attrs->{view} );
-        $c->log->debug(
-            "C::V::Email::Template uses specified view $view for rendering.")
-          if $c->debug;
+    if (exists $attrs->{view} && defined $attrs->{view} && $attrs->{view} ne '') {
+        $view = $c->view($attrs->{view});
+        $c->log->debug("C::V::Email::Template uses specified view $view for rendering.") if $c->debug;
     }
-
     # if none specified use the configured default view
     elsif ($default_view) {
         $view = $c->view($default_view);
-        $c->log->debug(
-            "C::V::Email::Template uses default view $view for rendering.")
-          if $c->debug;
+        $c->log->debug("C::V::Email::Template uses default view $view for rendering.") if $c->debug;;
     }
-
     # else fallback to Catalysts default view
     else {
         $view = $c->view;
-        $c->log->debug(
-"C::V::Email::Template uses Catalysts default view $view for rendering."
-        ) if $c->debug;
+        $c->log->debug("C::V::Email::Template uses Catalysts default view $view for rendering.") if $c->debug;;
     }
 
     # validate the per template view
     $self->_validate_view($view);
-
+    
     # prefix with template_prefix if configured
-    my $template =
-      $template_prefix ne ''
-      ? join( '/', $template_prefix, $attrs->{template} )
-      : $attrs->{template};
-
+    my $template = $template_prefix ne '' ? join('/', $template_prefix, $attrs->{template}) : $attrs->{template};
+   
     # setup the attributes (merge with defaults)
-    my $e_m_attrs = $self->SUPER::setup_attributes( $c, $attrs );
+    my $e_m_attrs = $self->setup_attributes($c, $attrs);
 
     # render the email part
-    my $output = $view->render(
-        $c,
-        $template,
-        {
-            content_type => $e_m_attrs->{content_type},
-            stash_key    => $self->stash_key,
-            %{$c->stash},
-        }
-    );
+    my $output = $view->render( $c, $template, { 
+        content_type    => $e_m_attrs->{content_type},
+        stash_key       => $self->{stash_key},
+        %{$c->stash},
+    });
+    
     if ( ref $output ) {
         croak $output->can('as_string') ? $output->as_string : $output;
     }
@@ -230,65 +192,56 @@ sub generate_part {
     );
 }
 
-=item process
+sub process {
+    my ( $self, $c ) = @_;
 
-The process method is called when the view is dispatched to. This creates the
-multipart message and then sends the message contents off to
-L<Catalyst::View::Email> for processing, which in turn hands off to
-L<Email::Sender::Simple>.
+    # don't validate template_prefix
 
-=cut
+    # the default view is validated if used
 
-around 'process' => sub {
-    my ( $orig, $self, $c, @args ) = @_;
-    my $stash_key = $self->stash_key;
-    return $self->$orig( $c, @args )
-      unless $c->stash->{$stash_key}->{template}
-          or $c->stash->{$stash_key}->{templates};
+    # the content type should be validated by Email::MIME::Creator
+    
+    my $stash_key = $self->{stash_key};
 
+    croak "No template specified for rendering"
+        unless $c->stash->{$stash_key}->{template}
+            or $c->stash->{$stash_key}->{templates};
+    
+    # this array holds the Email::MIME objects
     # in case of the simple api only one
-    my @parts = ();
+    my @parts = (); 
 
     # now find out if the single or multipart api was used
     # prefer the multipart one
-
+    
     # multipart api
-    if (   $c->stash->{$stash_key}->{templates}
+    if ($c->stash->{$stash_key}->{templates}
         && ref $c->stash->{$stash_key}->{templates} eq 'ARRAY'
-        && ref $c->stash->{$stash_key}->{templates}[0] eq 'HASH' )
-    {
-
+        && ref $c->stash->{$stash_key}->{templates}[0] eq 'HASH') {
         # loop through all parts of the mail
-        foreach my $part ( @{ $c->stash->{$stash_key}->{templates} } ) {
-            push @parts,
-              $self->generate_part(
-                $c,
-                {
-                    view         => $part->{view},
-                    template     => $part->{template},
-                    content_type => $part->{content_type},
-                    charset      => $part->{charset},
-                }
-              );
+        foreach my $part (@{$c->stash->{$stash_key}->{templates}}) {
+            push @parts, $self->generate_part($c, {
+                view            => $part->{view},
+                template        => $part->{template},
+                content_type    => $part->{content_type},
+                charset         => $part->{charset},
+            });
         }
     }
-
     # single part api
-    elsif ( $c->stash->{$stash_key}->{template} ) {
-        push @parts,
-          $self->generate_part( $c,
-            { template => $c->stash->{$stash_key}->{template}, } );
+    elsif($c->stash->{$stash_key}->{template}) {
+        push @parts, $self->generate_part($c, {
+            template    => $c->stash->{$stash_key}->{template},
+        });
     }
-
+    
     delete $c->stash->{$stash_key}->{body};
     $c->stash->{$stash_key}->{parts} ||= [];
-    push @{ $c->stash->{$stash_key}->{parts} }, @parts;
-    
-    return $self->$orig($c);
+    push @{$c->stash->{$stash_key}->{parts}}, @parts;
 
-};
-
-=back
+    # Let C::V::Email do the actual sending.  We just assemble the tasty bits.
+    return $self->next::method($c);
+}
 
 =head1 TODO
 
@@ -321,7 +274,7 @@ J. Shirley <jshirley@gmail.com>
 
 Simon Elliott <cpan@browsing.co.uk>
 
-Alexander Hartmaier <abraxxa@cpan.org>
+Alexander Hartmaier <alex_hartmaier@hotmail.com>
 
 =head1 LICENSE
 
